/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: uart.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

/*
UART library. Version 0.3
www.tobias-schlegel.de
*/

#include <avr/io.h>
#include "uart.h"


uint8_t uartstate = 0;						//UART Statusvariable



uint8_t init_uart(uint16_t ubrr){	 		//UART initialisieren

	UBRR0L = (uint8_t) (ubrr & 0xFF);		//Baudrate
	UBRR0H = (uint8_t) ((ubrr & 0xF00) >> 8);
	UCSR0B |= ((1<<TXEN0) | (1<<RXEN0));		//Tx- und RxEnable
	UCSR0C |= ((1<<UPM00) | (3<<UCSZ00));		//Parity, STOP usw...

	uartstate = 1;							//Statusvariable: UART offen (!= 0).

	return 0;
}


uint8_t uart_tx_ready(void){									//Uart bereit zeichen zu senden?

	if(uartstate != 0){													//UART initialisiert?
		if((UCSR0A & (1<<UDRE0)) ){									//Ist UART frei? nein dann warten wir halt.
            return 1;
		}else{
            return 0;
        }
	}else{																//UART nicht frei? oops
		return 0;														//ERROR
	}

	return 0;
}



uint8_t uart_rx_ready(void){											//UART bereit Zeichen zu empfangen.

	if(uartstate != 0){													//UART initialisiert?
		if(UCSR0A & (1<<RXC0)){
		    return 1;
        }else{
            return 0;
        }
	}else{																//UART nicht frei? oops
		return 0;														//ERROR
	}

    return 0;
}


uint8_t uart_put_array(uint8_t * array, uint8_t length){				//Sendet einen String (max 255 zeichen).
	uint8_t currentchar = 0;											//Variablen

	if(uartstate != 0){													//UART-Status.
		for(currentchar = 0 ; currentchar < length ; currentchar++){	//String-Schlefe

			while(!(UCSR0A & (1<<UDRE0)));								//berprfen ob der UART was hat, gegebenenfalls warten
			UDR0 = (uint8_t) array[currentchar];						//Senden, zur sicherheit noch in BYTE umwandeln.

		}
	}else{																//UART nicht initialisiert?
		return 1;														//ERROR!!
	}

	return 0;
}


uint8_t uart_putcs(const char string[255], uint8_t length){				//Sendet einen String (max 255 zeichen).
	uint8_t currentchar = 0;											//Variablen

	if(uartstate != 0){													//UART-Status.
		for(currentchar = 0 ; currentchar < length ; currentchar++){	//String-Schlefe

			while(!(UCSR0A & (1<<UDRE0)));								//berprfen ob der UART was hat, gegebenenfalls warten
			UDR0 = (uint8_t) string[currentchar];						//Senden, zur sicherheit noch in BYTE umwandeln.

		}
	}else{																//UART nicht initialisiert?
		return 1;														//ERROR!!
	}

	return 0;
}



uint8_t uart_puts(const char *string){									//Sendet einen String
	uint8_t currentchar = 0;											//Variablen

	if(uartstate != 0){													//UART-Status.
		while((uint8_t)string[currentchar] != 0){						//String-Schlefe

			while(!(UCSR0A & (1<<UDRE0)));								//berprfen ob der UART was hat, gegebenenfalls warten
			UDR0 = (uint8_t) string[currentchar];						//Senden, zur sicherheit noch in BYTE umwandeln.

			currentchar++;
		}
	}else{																//UART nicht initialisiert?
		return 1;														//ERROR!!
	}

	return 0;
}



uint8_t uart_putc(uint8_t letter){									//Wir senden ein einzelnes Zeichen

	if(uartstate != 0){													//UART initialisiert?
		while(!(UCSR0A & (1<<UDRE0)));									//Ist UART frei? nein dann warten wir halt.
		UDR0 = (uint8_t) letter;											//Zeichen senden
	}else{																//UART nicht frei? oops
		return 1;														//ERROR
	}

	return 0;
}



uint8_t uart_getc(void){												//UART Zeichen empfangen.

	if(uartstate != 0){													//UART initialisiert?
		while(!(UCSR0A & (1<<RXC0)));										//Ist UART frei? nein dann warten wir halt.
		return UDR0;														//Zeichen senden
	}else{																//UART nicht frei? oops
		return 0;														//ERROR
	}

}



uint8_t uart_gets(volatile char * string, uint8_t ov){					//UART Zeichen empfangen.

	uint8_t			i = 0;
	char			c;

	if(uartstate != 0){													//UART initialisiert?

		while(!(UCSR0A & (1<<RXC0)));										//Ist UART frei? nein dann warten wir halt.
		c = UDR0;

		while((c != (char) 0) && (i < ov)){
			string[i] = c;
			i++;

			while(!(UCSR0A & (1<<RXC0)));									//Ist UART frei? nein dann warten wir halt.
			c = UDR0;

		}

		if(c == (char) 0){
			return 1;
		}else{
			return 2;
		}

	}else{																//UART nicht frei? oops
		return 0;														//ERROR
	}

	return 255;
}


uint8_t uart_getstate(void){											//UART Status feststellen

	if(uartstate != 0){													//UART initialisiert?
		return 0;														//return JA
	}else{																//UART nicht frei? oops
		return 1;														//return NEIN
	}

}


