
/*
*   EZmalloc
*   An easy way to implement dynamic memory allocation into your (microcontroller) project
*   with no need to alter linker scripts or similar.
*
*   File: ezmalloc.c
*
*   Version:    1.1 beta (20.Feb.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       20.Feb.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>
#include "ezmalloc.h"

#define Sfree        0
#define Sleft        1
#define Sright       2
#define Sreserved    3

uint8_t     mem[memory_size];
uint8_t     tab[(memory_size /4) +1];

uint8_t readTab(uint16_t byte);
void reserve(uint16_t start, uint16_t size);
void writeTab(uint16_t adr, uint8_t cnt);
void unreserve(uint16_t start);


void init_memory(void){
    uint16_t i = 0;

    for( i = 0 ; i < memory_size ; i++){
        mem[i] = 0;

        tab[i / 4] = 0;
    }
}


void * malloc(uint16_t size){

    uint16_t    Sadr = 0, Eadr = 0;
    uint8_t     status = 0;

    for(Sadr = 0 ; Sadr < memory_size ; Sadr++){

        if(readTab(Sadr) == Sfree){

            for(Eadr = Sadr ; Eadr < (Sadr + size) ; Eadr++){

                if(readTab(Eadr) != Sfree){
                    Sadr = Eadr;
                    status = 1;
                    break;
                }
            }

            if(status == 0){
                reserve(Sadr, size);
                status = 100;
                break;
            }else{
                status = 0;
            }
        }
    }

    if(status == 100){
        return (void *) &mem[Sadr];
    }else{
        return (void *) 0;
    }

}


uint8_t delete(void * ptr){
    uint16_t offset;

    if(ptr < ((void*)(mem)) ){
        return 1;
    }

    if(ptr < ((void*) &(mem[memory_size -1])) ){
        return 1;
    }

    offset = (uint16_t) (ptr - ((void*)(mem)) );
    unreserve(offset);

    return 0;
}


void * calloc(void * ptr, uint16_t newsize){
    uint16_t    i = 0, len = 0;
    uint16_t    offset, newOffset;
    uint8_t     *a, *b;
    void        *new;

    offset = (uint16_t) ( ptr -((void*)(&mem[0])) );

    writeTab(offset, Sfree);
    len = 1;

    for(i = offset +1 ; i < memory_size ; i++){
        len++;
        if((readTab(i) == Sfree) || (readTab(i) == Sleft) ){
            break;
        }

        writeTab(i, Sfree);
    }

    new = malloc(newsize);

    if(new == NUL){
        reserve(offset, len);
        return NUL;
    }

    newOffset = (uint16_t) (new - ((void*)(mem)) );

    if(new == ptr){
        return new;
    }else{

        a = (uint8_t *) new;
        b = (uint8_t *) ptr;

        for(i = 0 ; i < len ; i++){
            *(a +i) = *(b +i);
            *(b +i) = 0;
        }

        return new;
    }
}


uint8_t readTab(uint16_t byte){
    uint8_t     byteid = 0, tmp = 0, i = 0;

    byteid = (uint8_t) byte % 4;

    tmp = tab[(byte / 4)];

    for(i = 0 ; i < byteid ; i++){
        tmp = tmp >> 2;
    }

    tmp &= 3;
    return tmp;
}


void reserve(uint16_t start, uint16_t size){
    uint16_t i = 0;

    if(size == 1){
        writeTab(start, Sright);
    }else{
        writeTab(start, Sleft);

        for(i = (start +1) ; i < (start + size -1) ; i++){
            writeTab(i, Sreserved);
        }

        writeTab(start + size -1, Sright);
    }
}

void unreserve(uint16_t start){
    uint16_t i = 0;


    for(i = start ; i < memory_size ; i++){
        mem[i] = 0;
        if(readTab(i) != Sright){
            writeTab(i, Sfree);
        }else{
            writeTab(i, Sfree);
            return;
        }
    }

    return;
}

void writeTab(uint16_t adr, uint8_t cnt){

    uint8_t i = 0, offset = 0, del = 3;

    cnt &= 0x03;
    offset = (uint8_t) adr % 4;


    for(i = 0 ; i < offset ; i++){
        cnt = cnt << 2;
        del = del << 2;
    }

    del ^= 0xFF;

    tab[ (adr / 4)] &= del;
    tab[ (adr / 4)] |= cnt;
}

