
/*
*   LibInterface
*   A very basic hardware abstraction library.
*   -> Main source file
*
*   File: libinterface.c
*
*   Version:    1.0
*   File-Ver.:  1.1 (23.Feb.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       5.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "libinterface.h"

iface       iFaces[if_INTERFACE_TABLE_SIZE];
uint8_t     if_Count = 0;
/**
    initializes the interface table
    @return if_OK
*/
uint8_t     if_init(void){

    uint8_t     i = 0;

    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        iFaces[i].Conf = if_UNUSED;
    }

    if_Count = 0;

    return if_OK;
}

/**
    Creates a new interface
    @param[]    id              ID of the interface to create
    @param[]    config          Interface configuration
    @param[]    address         Write-offset address (see IF-config)
    @param[]    target          Pointer to target (if memory target)
    @param[]    target_size     maximum write size for target (in n * if_datatype)
    @param[]    wr              Pointer to Driver-write-function
    @param[]    rd              ...read-function
    @param[]    ctl             ...control-function
    @return     if_OK or errorcode
*/
uint8_t     if_createInterface(uint8_t id, uint8_t config, uint16_t address, if_datatype * target, uint8_t target_size, if_write wr, if_read rd, if_control ctl){

    uint8_t     i = 0;

    #if if_ALLOW_MULTIPLE_WRITE
    #else
    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){

        if(iFaces[i].Id == id){
            return if_MULTIPLE_CREATE;
        }
    }

    #endif

    if(config == if_UNUSED){
        return if_BAD_CONFIG;
    }

    if(ctl == if_NO_CONTROL){
        return if_NO_CONTROL_TARGET;
    }

    if( (!(config & if_WRITEONLY)) && (rd == if_NO_READ) ){
        return if_NO_READ_TARGET;
    }

    if( (!(config & if_READONLY)) && (wr == if_NO_WRITE) ){
        return if_NO_WRITE_TARGET;
    }

    if( (config & if_MEMORY_TARGET)){
        if(target == if_NO_TARGET){
            return if_NO_MEMORY_TARGET;
        }

        if(target_size == 0){
            return if_TARGET_SIZE_ZERO;
        }
    }

    if(if_Count == if_INTERFACE_TABLE_SIZE - 1){
        return if_IF_TABLE_FULL;
    }

    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( (iFaces[i].Conf == if_UNUSED)){
            iFaces[i].Conf = (if_ACTIVE | config);
            iFaces[i].Target = target;
            iFaces[i].TargetSize = target_size;
            iFaces[i].Address = address;
            iFaces[i].Id = id;
            iFaces[i].Write = wr;
            iFaces[i].Read = rd;
            iFaces[i].Control = ctl;

            if_Count++;
            return if_OK;
            break;
        }
    }

    return if_IF_TABLE_FULL;

}

/***/
uint8_t     if_deleteInterface(uint8_t id){

    uint8_t     i = 0;
    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     found = 0;
    #endif

    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){
            iFaces[i].Conf = if_UNUSED;

            #if if_ALLOW_MULTIPLE_WRITE
            found++;
            #else
            return if_OK;
            #endif
        }
    }

    #if if_ALLOW_MULTIPLE_WRITE
        if(found > 0){
            return if_OK;
        }
    #endif

    return if_NOT_FOUND;
}

/***/
uint8_t     if_lockInterface(uint8_t id){

    uint8_t     i = 0;
    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     count = 0;
    #endif


    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){
            iFaces[i].Conf |= if_LOCKED;
            #if if_ALLOW_MULTIPLE_WRITE
            count++;
            #else
            return if_OK;
            #endif
        }
    }

    #if if_ALLOW_MULTIPLE_WRITE
    if(count > 0){
        return if_OK;
    }else{
        return if_NOT_FOUND;
    }
    #else
    return if_NOT_FOUND;
    #endif
}

/***/
uint8_t     if_unlockInterface(uint8_t id){

    uint8_t     i = 0;
    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     count = 0;
    #endif


    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){
            iFaces[i].Conf &= ~if_LOCKED;
            #if if_ALLOW_MULTIPLE_WRITE
            count++;
            #else
            return if_OK;
            #endif
        }
    }

    #if if_ALLOW_MULTIPLE_WRITE
    if(count > 0){
        return if_OK;
    }else{
        return if_NOT_FOUND;
    }
    #else
    return if_NOT_FOUND;
    #endif
}


/**
    Writes to an interface (or several) with the same id
    @param[in]      id      the Interface ID (uint8_t)
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to write. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
    @return depends on mode: \li \c Multiwrite: last errorcode; *data points on the amount of successful writes
    \li \c Normal: if_OK if write was successful or error code
*/
uint8_t     ifWrite(uint8_t id, uint16_t adr, if_datatype * data, uint8_t len){
    uint8_t     i = 0, j = 0, res = 0;
    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     err = if_OK, count = 0;
    #endif

    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){

            if(iFaces[i].Conf & if_LOCKED){
                #if if_ALLOW_MULTIPLE_WRITE
                err = if_READONLY;
                #else
                return if_IF_LOCKED;
                #endif
            }else{

                if( !(iFaces[i].Conf & if_ONLINE) ){
                    #if if_ALLOW_MULTIPLE_WRITE
                    err = if_READONLY;
                    #else
                    return if_OFFLINE;
                    #endif
                }else{

                    if(iFaces[i].Conf & if_READONLY){
                        #if if_ALLOW_MULTIPLE_WRITE
                        err = if_READONLY;
                        #else
                        return if_READONLY;
                        #endif
                    }else{

                        if(iFaces[i].Conf & if_MEMORY_TARGET){

                            if(len > iFaces[i].TargetSize){
                                return if_WRITE_SIZE_LIMIT;
                            }

                            for(j = 0 ; j < len ; j++){

                                if(iFaces[i].Target != 0){
                                    *(iFaces[i].Target + j) = *(data+j);
                                }else{
                                    #if if_ALLOW_MULTIPLE_WRITE
                                    err = if_NO_MEMORY_TARGET;
                                    #else
                                    return if_NO_MEMORY_TARGET;
                                    #endif
                                }
                            }
                            #if if_ALLOW_MULTIPLE_WRITE
                            count++;
                            #endif
                        }else{
                            if(iFaces[i].Write == (if_write) 0){
                                #if if_ALLOW_MULTIPLE_WRITE
                                err = if_NO_WRITE_TARGET;
                                #else
                                return if_NO_WRITE_TARGET;
                                #endif
                            }

                            if(iFaces[i].Conf & if_FIXED_ADDRESS){
                                res = (*(iFaces[i].Write))(iFaces[i].Address, data, len);
                            }else{
                                res = (*(iFaces[i].Write))(adr, data, len);
                            }

                            if(res != if_OK){
                                #if if_ALLOW_MULTIPLE_WRITE
                                err = if_DRIVER_FUNCTION_ERROR;
                                #else
                                return if_DRIVER_FUNCTION_ERROR;
                                #endif
                            }
                            #if if_ALLOW_MULTIPLE_WRITE
                            else{
                                count++;
                            }
                            #endif
                        }
                    }

                }
            }

            #if if_ALLOW_MULTIPLE_WRITE
            data = (if_datatype *) count;
            return err;
            #else
            return if_OK;
            #endif
        }
    }

    #if if_ALLOW_MULTIPLE_WRITE
    if(err != 0){
        return err;
    }else{
        return if_OK;
    }
    #else
    return if_NOT_FOUND;
    #endif
}

/**
    Reads from an interface. If multiwrite engaged, it reads from the interface configed als if_READ_FROM_MULTI
    or from the first in the list...
    @param[in]      id      the Interface ID (uint8_t)
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to space for data to read. Can be used as two-way communication
    @param[in]      len     Amount of units to read (in n * if_datatype)
    @return \li \c if_OK if successful \li \c errorcode for anything else
*/
uint8_t     ifRead(uint8_t id, uint16_t adr, if_datatype * data, uint8_t len){
    uint8_t     i = 0, res = 0, use = 255;

    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     count = 0;
    #endif


    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){

            if(use == 255){
                use = i;
            }

            #if if_ALLOW_MULTIPLE_WRITE
            count++;

            if(iFaces[i].Conf & if_READ_FROM_MULTI){
                use = i;
                break;
            }
            #else
            break;
            #endif
        }
    }

    if(use < 255){

        if( !(iFaces[use].Conf & if_ONLINE)){
            return if_OFFLINE;
        }

        if(iFaces[use].Conf & if_LOCKED){
            return if_IF_LOCKED;
        }

        if(iFaces[use].Read != if_NO_READ){

            if(!(iFaces[use].Conf & if_FIXED_ADDRESS)){
                res = (*(iFaces[use].Read))(adr, data, len);
            }else{
                res = (*(iFaces[use].Read))(iFaces[use].Address, data, len);
            }

            if(res != if_OK){
                return if_DRIVER_FUNCTION_ERROR;
            }else{
                return if_OK;
            }
        }
    }else{
        return if_NOT_FOUND;
    }

    return if_SHOOT_ME;
}


/**
    Control function acty als control bridge for interfaces. \n
    Note: if the control-word (ctl) is 0 the control-function must shutdown the IF, if the word is 1 it must initialize it.
    @param[in]      id      the Interface ID (uint8_t)
    @param[in]      ctl     control-word
    @param[in,out]  * data  Pointer to control-data. Can be used as two-way communication
    @return \li \c Multiwrite if_OK if more than 0 controls have been written or errorcode \li \c Normal ifOK or errorcode
*/
uint8_t     ifControl(uint8_t id, uint16_t ctl, if_datatype * data){
    uint8_t     i = 0, res = 0;
    #if if_ALLOW_MULTIPLE_WRITE
    uint8_t     count = 0;
    #endif


    for(i = 0 ; i < if_INTERFACE_TABLE_SIZE ; i++){
        if( iFaces[i].Id == id){
            res = (*(iFaces[i].Control))(ctl, data);

            if( (ctl == 0) && (res == if_OK) ){
                iFaces[i].Conf &= ~if_ONLINE;
            }

            if( (ctl == 1) && (res == if_OK) ){
                iFaces[i].Conf |= if_ONLINE;
            }

            #if if_ALLOW_MULTIPLE_WRITE
            count++;
            #else
            return if_OK;
            #endif
        }
    }

    #if if_ALLOW_MULTIPLE_WRITE
    if(count > 0){
        return if_OK;
    }else{
        return if_NOT_FOUND;
    }
    #else
    return if_NOT_FOUND;
    #endif
}

