
/*
*   MenuLib
*   A library for generating simple tree based menus on small (embedded) systems.
*   Download a demo application from www.tobias-schlegel.de
*
*   File: menu.c
*
*   Version:    1.0 (2.Jan.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       2.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "menu.h"
#include "string.h"

volatile uint8_t    mLayer = 0;
menudata            * mMenu;
volatile uint8_t    mMenuLen = 0;
menudata            * mActualMenuElement;

void init_menu(menudata * menu, uint8_t entries){
    mMenu = menu;
    mActualMenuElement = &(mMenu[1]);
    mMenuLen = entries;
}

uint8_t getChild(uint8_t parent){
    uint8_t sum = 0, i = 0;

    for(i = 0 ; i < parent ; i++){
        sum += mMenu[i].children;
    }

    if(sum > mMenuLen){
        return parent;
    }

    return (sum+1);
}

uint8_t getParent(uint8_t child){
    uint8_t sum = 0, i = 0;

    for(i = 0 ; i < mMenuLen ; i++){
        sum += mMenu[i].children;

        if((child <= sum) && (child > (sum - mMenu[i].children))){
            return i;
        }
    }
    return child;
}

uint8_t getIndex(menudata * ptr){
    uint8_t i = 0;

    for(i = 0 ; i < mMenuLen+1 ; i++){
        if(ptr == &mMenu[i]){
            return i;
        }
    }
    return 0;
}

#ifdef mOffsetString
void addOffset(char * t, uint8_t offset, uint8_t payload, uint8_t maxlen, const char * cOffset){
    uint8_t i = 0, real;

    real = offset + payload;

    if(real > maxlen){
        real = maxlen - payload;
    }else{
        real = offset;
    }

    if(real > 0){
        if(real != offset){
            str_cpy(t, (char *)mOffsetShortenedString);
        }else{
            str_cpy(t, (char *)cOffset);
        }

        for(i = 1 ; i < real ; i++){
            str_cat(t, (char *)cOffset);
        }
    }
}
#endif

void mGetText(char * txt, uint8_t lines, uint8_t linelen, uint8_t parents){
    uint8_t actual = 0, b4 = 0, aft = 0, showentries = 0;
    uint8_t showmax = 0, start = 0, stop = 0, parent = 0, parentcount =  0;
    uint8_t i = 0, space = 0 ;
    uint8_t line = 0, offset = 1, tmp = 0;

    for(i = 0 ; i < (lines * linelen +1) ; i++){
        txt[i] = (char) 0;
    }

    space = linelen +1;
    actual = getIndex(mActualMenuElement);
    start = getChild(getParent(actual));
    stop = start + mMenu[getParent(actual)].children - 1;
    showmax = mMenu[getParent(actual)].children;

    if(parents > 0){ //mode showing parent menu in first lines
        if(lines < parents){
            parents = lines -1;
        }

        parent = actual;

        for(i = 0 ; i < mMenuLen ; i++){
            if(parent == 0){
                parentcount = i;
                break;
            }
            parent = getParent(parent);
        }

        if(parentcount < parents){
            parents = parentcount;
        }

        if(parents > (lines -1)){
            parents = lines -1;
        }

        parent = actual;
        tmp = parents-1;

        for(i = 0 ; i < parents ; i++){
            #ifdef mOffsetString
            addOffset(&(txt[tmp * space]), tmp, str_len(mMenu[getParent(parent)].cText), linelen, mOffsetString);
            #endif
            str_cat(&(txt[tmp * space]), mMenu[getParent(parent)].cText);
            parent = getParent(parent);
            line++;
            tmp--;
            offset++;
        }
    }

    showentries = (lines - line);

    if(showentries > 1){
        if(actual == start){
            b4 = stop;
        }else{
            b4 = actual -1;
        }

        if(actual == stop){
            aft = start;
        }else{
            aft = actual + 1;
        }

        #ifdef mOffsetString
        addOffset(&(txt[line * space]), offset-1, str_len(mMenu[b4].cText), linelen, mOffsetString);
        #endif
        #ifdef mNotMarkedString
        str_cat(&(txt[line * space]), mNotMarkedString);
        #endif
        str_cat(&(txt[line * space]), mMenu[b4].cText);
        line++;
    }

    //add marked menu entry.
    #ifdef mOffsetString
    addOffset(&(txt[line * space]), offset-1, str_len(mMenu[actual].cText), linelen, mOffsetString);
    #endif
    str_cat(&(txt[line * space]), mMarkedString);
    str_cat(&(txt[line * space]), mMenu[actual].cText);
    line++;

    showentries = (lines - line);

    if(showentries > 0){
        for(i = 0 ; i < showentries ; i++){
            #ifdef mOffsetString
            addOffset(&(txt[line * space]), offset -1, str_len(mMenu[aft+i].cText), linelen, mOffsetString);
            #endif
            #ifdef mNotMarkedString
            str_cat(&(txt[line * space]), mNotMarkedString);
            #endif
            str_cat(&(txt[line * space]), mMenu[aft+i].cText);
            line++;
        }
    }
}

void mStatemachine(uint8_t key){
    uint8_t start = 0, stop = 0, actual = 0;

    actual = getIndex(mActualMenuElement);
    start = getChild(getParent(actual));
    stop = start + mMenu[getParent(actual)].children - 1;

    if(key == mKeyUp){  //nav up (scroll)
        if((actual == start) || (actual > stop) || (actual < start) ){
            mActualMenuElement = &(mMenu[stop]);
        }else{
            mActualMenuElement = &(mMenu[actual -1]);
        }
        return;
    }

    if(key == mKeyDown){ //nav down (scroll)
        if((actual == stop) || (actual > stop) || (actual < start) ){
            mActualMenuElement = &(mMenu[start]);
        }else{
            mActualMenuElement = &(mMenu[actual +1]);
        }
        return;
    }

    if(key == mKeyOK){ //one layer deeper in the structure or Callback
        if(mActualMenuElement->children > 0){
            mActualMenuElement = &(mMenu[getChild(getIndex(mActualMenuElement))]);
        }else{
            if(mActualMenuElement->CallBack > 0){
                #ifdef mCallbackWithParams
                mActualMenuElement->CallBack((mActualMenuElement->CallBackArgument));
                #else
                mActualMenuElement->CallBack();
                #endif
            }
        }
        return;
    }

    if(key == mKeyBack){ //one layer up.
        mActualMenuElement = &(mMenu[getParent(getIndex(mActualMenuElement))]);
        return;
    }
    return;
}

